# Extracting from files

To do this exercice you will need to download Frech First name data from "Institut national de la statistique
et des études économiques"

```bash
wget https://www.insee.fr/fr/statistiques/fichier/2540004/nat2021_csv.zip
unzip nat2021_csv.zip 
```

You should now have a file called `nat2021.csv` in your working directory.

## Displaying sample (head, tail)

When you have a huge dataset, it can be interesting to only display the beginning or the end of the file, to have an idea of how the file is structured.
Using commands `head` and `tail` allows to do this tasks.

!!! question "Display the first 20 lines of `nat2021.csv` file"

??? example "Click to show the solution"  
    ```bash
    head -n 20 nat2021.csv
    ```

!!! question "Display the last 10 lines of `nat2021.csv` file"

??? example "Click to show the solution"  
    ```bash
    tail -n 10 nat2021.csv
    ```

## Counting words/lines (wc)

!!! question "Count the number of characters of `nat2021.csv` file"

??? example "Click to show the solution"  
    ```bash
    wc -c nat2021.csv
    ```

!!! question "Count the number of line of `nat2021.csv` file"

??? example "Click to show the solution"  
    ```bash
    wc -l nat2021.csv
    ```

## Searching patterns (grep)

!!! question "Select all line related of the year 2001 in `nat2021.csv` file"

??? example "Click to show the solution"  
    ```bash
    grep ";2021;" nat2021.csv
    ```

!!! question "How many names have been provided in 2021?"

??? example "Click to show the solution"  
    ```bash
    grep ";2021;" nat2021.csv | wc -l
    # result: 13501
    ```

!!! question "Is there more diversity in male or female names in 2021"?

??? example "Click to show the solution"  
    ```bash
    # female
    grep ";2021;" nat2021.csv | grep "^2" | wc -l
    # result: 7112
    # male
    grep ";2021;" nat2021.csv | grep "^1" | wc -l
    # result: 6389
    ```

!!! question "How many person are called PARIS in 2021"?

??? example "Click to show the solution"  
    ```bash
    # female
    grep "PARIS;2021;" nat2021.csv
    # result 16 (5 male and 11 female)
    ```

The rare name ([see here for documentation](https://www.insee.fr/fr/statistiques/2540004?sommaire=4767262#documentation)) are set as `_PRENOMS_RARES`.

!!! question "Could you find all rare name ? Do you see any pattern?"

??? example "Click to show the solution"  
    ```bash
    grep ";_PRENOMS_RARES;" nat2021.csv
    ```
    People tends to provide more and more rare names.


## Sorting a tabular file (sort)

It is possible to sort a file or tabulated output using the `sort` command:

```bash
sort nat2021.csv | head
```

Sort is particularly useful when you use some key options:  

 * `-n` to sort numerically  
 * `-t` to specify a separator (the default separator is a space or a tab)  
 * `-k` to specify on which column you want to sort the lines (use together with `-t`)  

Try the numerical sort. 

```bash
sort -n nat2021.csv | head
```

!!! question "Do you observe any difference?"


!!! question "What year and what name has been the most provided among the records?"

??? example "Click to show the solution"  
    ```bash
    # command
    sort -n -t ';' -k4  nat2021.csv
    # result: JEAN
    ```

!!! question "What year was the most prolific fot the name ZINEDINE?"

??? example "Click to show the solution"  
    ```bash
    # command
    grep ";ZINEDINE;" nat2021.csv | sort -n -t ';' -k4
    # result: 1998
    ```

## Extracting columns (cut)

The `cut` command allows to cut a line at a specific character and extract a selected field: `cut -d";" -f 2`  

 * `-d` specify the separator  
 * `-f` specify the field to extract   

!!! question "How to extract the name of the top 100 names/year the most provided"

??? example "Click to show the solution"  
    ```bash
    # command
    sort -n -t ';' -k4  nat2021.csv | tail -n 100 | cut -d";" -f 2
    ```

The `uniq` command can be used to remove the redundancy. But result need to be sorted to make it work properly/

!!! question "Could you now find a way to filter the redundancy"

??? example "Click to show the solution"  
    ```bash
    # command
    sort -n -t ';' -k4  nat2021.csv | tail -n 100 | cut -d";" -f 2 | sort | uniq
    ```

!!! question "How many time the name JEAN has been provided in total?"

??? example "Click to show the solution"  
    It start to be too complicated for the command you have seen so far, you need to use a command specific to column data `awk`


## Redirecting an output (>)

You can redirect a result and store it in a file thanks to the `>` redirection:  
`command > filename`

!!! question "Save all the names from 2005 in a dedicated file?"

??? example "Click to show the solution"  
    ```bash
    # command
    grep ";2025;" nat2021.csv > names2005.txt
    ```


## Filtering a file (awk)



## Replacing patterns (sed)



## Combining commands (| && ;)





