# The Loops

Loops in Bash allow you to repeat a set of commands multiple times. Bash provides three main types of loops: For, While and Until.


## The different loops

### For

A for loop iterates over a list of items or a sequence of numbers.

* Looping over a list

```
#!/bin/bash
for name in Peter Paul Jack; do
    echo "Hello, $name!"
done
```

* Looping over a list of items - the case of associative array

```
#!/bin/bash
for key in ${!myAArray[@]} ; do
    echo "key: $key"
    echo "value: ${myAArray[$key]}"
done
```

*  Looping over a range of numbers

```
#!/bin/bash
for i in {1..5}; do
    echo "Iteration $i"
done
```

*  Looping over a range of numbers using C-style approach

```
#!/bin/bash
for (( i=1 ; i<=5 ; i++ )); do
    echo "Iteration $i"
done
```

### While

A while loop executes commands as long as a condition is true.

```bash
#!/bin/bash
count=1
while (( $count <= 5 )); do
    echo "Count: $count"
    ((count++))  # Increment the count
done
```

### Until

An until loop is the opposite of while: it runs until the condition is true

```bash
#!/bin/bash
count=1
until (( $count > 5 )); do
    echo "Count: $count"
    ((count++))  # Increment the count
done
```

## Real cases

### Looping Through Files

A common use case in Bash is iterating over files in a directory. 

```bash
#!/bin/bash
for file in *.txt; do
    echo "Processing $file"
done
```

This will list all `.txt` file in the current directory.

### Looping over a command's result

Using `$(command)` you can execute first run a command and provide its output to a list to iterate through.

```bash
#!/bin/bash
for file in $(ls); do
    echo "Processing $file"
done
```

!!! note
    By default the internal field separator (IFS) is the space. So to iterate through line that contains spaces you should set the `IFS` to carriage return: `IFS=$'\n'`.
    Try the command above before and after to see the difference.

### Looping through lines of a file

Set the internal field separator to new line separator `IFS=$'\n'`, then you can use on of this approach:

```bash
for line in $(cat file.txt);do
    code # $line will be the variable that contains the line
done
```
or 

```bash
while read line;do
    code # $line will be the variable that contains the line
done < file
```

## Breaking and Skipping in Loops

### break

The `break` command will exit the loop entirely

```bash
#!/bin/bash
for i in {1..5}; do
    if [[ $i -eq 3 ]]; then
        break
    fi
    echo "Processing $i"
done
```bash

### continue

The `continue` command will skip the current iteration and move to the next.

```bash
#!/bin/bash
for i in {1..5}; do
    if [[ $i -eq 3 ]]; then
        continue
    fi
    echo "Processing $i"
done
```bash